from pathlib import Path
from typing import Callable, ParamSpec, TypeVar

import click
from llama_deploy.core.config import DEFAULT_DEPLOYMENT_FILE_PATH

P = ParamSpec("P")
R = TypeVar("R")

# hack around for mypy not letting you set path_type=Path on click.Path
_ClickPath = getattr(click, "Path")


def _deployment_file_option(f: Callable[P, R]) -> Callable[P, R]:
    return click.argument(
        "deployment_file",
        required=False,
        default=DEFAULT_DEPLOYMENT_FILE_PATH,
        type=_ClickPath(dir_okay=True, resolve_path=True, path_type=Path),
    )(f)


def _python_version_option(f: Callable[P, R]) -> Callable[P, R]:
    return click.option(
        "--python-version",
        help="Python version for the base image. Default is inferred from the uv project configuration (.python-version or pyproject.toml). If no version can be inferred, python 3.12 is used.",
        required=False,
        default=None,
    )(f)


def _port_option(f: Callable[P, R]) -> Callable[P, R]:
    return click.option(
        "--port",
        help="The port to run the API server on. Defaults to 4501.",
        required=False,
        default=4501,
        type=int,
    )(f)


def _dockerignore_path_option(f: Callable[P, R]) -> Callable[P, R]:
    return click.option(
        "--dockerignore-path",
        help="Path for the output .dockerignore file. Defaults to .dockerignore",
        required=False,
        default=".dockerignore",
    )(f)


def _output_file_option(f: Callable[P, R]) -> Callable[P, R]:
    return click.option(
        "--output-file",
        help="Path for the output file to build the image. Defaults to Dockerfile",
        required=False,
        default="Dockerfile",
    )(f)


def _overwrite_option(f: Callable[P, R]) -> Callable[P, R]:
    return click.option(
        "--overwrite",
        help="Overwrite output files",
        is_flag=True,
    )(f)


def _exclude_option(f: Callable[P, R]) -> Callable[P, R]:
    return click.option(
        "--exclude",
        help="Path to exclude from the build (will be appended to .dockerignore). Can be used multiple times.",
        multiple=True,
        required=False,
        default=None,
    )(f)


def pkg_container_options(f: Callable[P, R]) -> Callable[P, R]:
    return _deployment_file_option(
        _python_version_option(
            _port_option(
                _dockerignore_path_option(
                    _overwrite_option(_exclude_option(_output_file_option(f)))
                )
            )
        )
    )



"""Configuration and profile management for llamactl"""

import functools
import os
import sqlite3
import uuid
from pathlib import Path
from typing import Any

from ._migrations import run_migrations
from .schema import DEFAULT_ENVIRONMENT, Auth, DeviceOIDC, Environment


def _serialize_device_oidc(value: DeviceOIDC | None) -> str | None:
    if value is None:
        return None
    return value.model_dump_json()


def _deserialize_device_oidc(value: str | None) -> DeviceOIDC | None:
    if not value:
        return None
    return DeviceOIDC.model_validate_json(value)


def _to_auth(row: Any) -> Auth:
    return Auth(
        id=row[0],
        name=row[1],
        api_url=row[2],
        project_id=row[3],
        api_key=row[4],
        api_key_id=row[5],
        device_oidc=_deserialize_device_oidc(row[6]),
    )


def _to_environment(row: Any) -> Environment:
    return Environment(
        api_url=row[0],
        requires_auth=bool(row[1]),
        min_llamactl_version=row[2],
    )


class ConfigManager:
    """Manages profiles and configuration using SQLite"""

    def __init__(self, init_database: bool = True):
        self.config_dir = self._get_config_dir()
        self.db_path = self.config_dir / "profiles.db"
        self._ensure_config_dir()
        if init_database:
            self._init_database()

    def _get_config_dir(self) -> Path:
        """Get the configuration directory path based on OS.

        Honors LLAMACTL_CONFIG_DIR when set. This helps tests isolate state.
        """
        override = os.environ.get("LLAMACTL_CONFIG_DIR")
        if override:
            return Path(override).expanduser()
        if os.name == "nt":  # Windows
            config_dir = Path(os.environ.get("APPDATA", "~")) / "llamactl"
        else:  # Unix-like (Linux, macOS)
            config_dir = Path.home() / ".config" / "llamactl"
        return config_dir.expanduser()

    def _ensure_config_dir(self) -> None:
        """Create configuration directory if it doesn't exist"""
        self.config_dir.mkdir(parents=True, exist_ok=True)

    def _init_database(self) -> None:
        """Initialize SQLite database and run migrations; then seed defaults."""

        with sqlite3.connect(self.db_path) as conn:
            # Apply ad-hoc SQL migrations based on PRAGMA user_version
            # Pass db_path to enable file-based locking across processes
            run_migrations(conn, self.db_path)

            conn.commit()

    def destroy_database(self) -> None:
        """Destroy the database"""
        self.db_path.unlink()
        self._init_database()

    #############################################
    ## Settings
    #############################################

    def set_settings_current_profile(self, name: str | None) -> None:
        """Set or clear the current active profile.

        If name is None, the setting is removed.
        """
        with sqlite3.connect(self.db_path) as conn:
            if name is None:
                conn.execute("DELETE FROM settings WHERE key = 'current_profile'")
            else:
                conn.execute(
                    "INSERT OR REPLACE INTO settings (key, value) VALUES ('current_profile', ?)",
                    (name,),
                )
            conn.commit()

    def get_settings_current_profile_name(self) -> str | None:
        """Get the name of the current active profile"""
        with sqlite3.connect(self.db_path) as conn:
            cursor = conn.execute(
                "SELECT value FROM settings WHERE key = 'current_profile'"
            )
            row = cursor.fetchone()
            return row[0] if row else None

    def set_settings_current_environment(self, api_url: str) -> None:
        """Set the current environment by URL.

        Requires the environment row to already exist (validated elsewhere, e.g. via
        a probe before creation). Raises ValueError if not found.
        """
        with sqlite3.connect(self.db_path) as conn:
            conn.execute(
                "INSERT OR REPLACE INTO settings (key, value) VALUES ('current_environment_api_url', ?)",
                (api_url,),
            )
            conn.commit()

    def create_profile(
        self,
        name: str,
        api_url: str,
        project_id: str,
        api_key: str | None = None,
        api_key_id: str | None = None,
        device_oidc: DeviceOIDC | None = None,
    ) -> Auth:
        """Create a new auth profile"""
        if not project_id.strip():
            raise ValueError("Project ID is required")
        profile = Auth(
            id=str(uuid.uuid4()),
            name=name,
            api_url=api_url,
            project_id=project_id,
            api_key=api_key,
            api_key_id=api_key_id,
            device_oidc=device_oidc,
        )

        with sqlite3.connect(self.db_path) as conn:
            try:
                conn.execute(
                    "INSERT INTO profiles (id, name, api_url, project_id, api_key, api_key_id, device_oidc) VALUES (?, ?, ?, ?, ?, ?, ?)",
                    (
                        profile.id,
                        profile.name,
                        profile.api_url,
                        profile.project_id,
                        profile.api_key,
                        profile.api_key_id,
                        _serialize_device_oidc(profile.device_oidc),
                    ),
                )
                conn.commit()
            except sqlite3.IntegrityError:
                raise ValueError(
                    f"Profile '{name}' already exists for environment '{api_url}'"
                )

        return profile

    def get_current_profile(self, env_url: str) -> Auth | None:
        """Get the current active profile"""
        current_name = self.get_settings_current_profile_name()
        if current_name:
            return self.get_profile(current_name, env_url)
        return None

    def get_current_environment(self) -> Environment:
        """Get the current active environment"""
        with sqlite3.connect(self.db_path) as conn:
            cursor = conn.execute(
                "SELECT value FROM settings WHERE key = 'current_environment_api_url'"
            )
            row = cursor.fetchone()
            api_url = row[0] if row else DEFAULT_ENVIRONMENT.api_url

            env_row = conn.execute(
                "SELECT api_url, requires_auth, min_llamactl_version FROM environments WHERE api_url = ?",
                (api_url,),
            ).fetchone()
            if env_row:
                return _to_environment(env_row)

        # Fallback: return an in-memory default without writing to DB
        if api_url == DEFAULT_ENVIRONMENT.api_url:
            return DEFAULT_ENVIRONMENT
        return Environment(
            api_url=api_url, requires_auth=False, min_llamactl_version=None
        )

    ##################################
    ## Profiles
    ##################################

    def get_profile(self, name: str, env_url: str) -> Auth | None:
        """Get a profile by name"""
        with sqlite3.connect(self.db_path) as conn:
            row = conn.execute(
                "SELECT id, name, api_url, project_id, api_key, api_key_id, device_oidc FROM profiles WHERE name = ? AND api_url = ?",
                (name, env_url),
            ).fetchone()
            if row:
                return _to_auth(row)
        return None

    def get_profile_by_id(self, id: str) -> Auth | None:
        """Get a profile by ID"""
        with sqlite3.connect(self.db_path) as conn:
            row = conn.execute(
                "SELECT id, name, api_url, project_id, api_key, api_key_id, device_oidc FROM profiles WHERE id = ?",
                (id,),
            ).fetchone()
            if row:
                return _to_auth(row)
        return None

    def get_profile_by_api_key(self, env_url: str, api_key: str) -> Auth | None:
        """Get a profile by api_key within an environment URL."""
        with sqlite3.connect(self.db_path) as conn:
            row = conn.execute(
                """
                SELECT id, name, api_url, project_id, api_key, api_key_id, device_oidc
                FROM profiles
                WHERE api_url = ? AND api_key = ?
                LIMIT 1
                """,
                (env_url, api_key),
            ).fetchone()
            if row:
                return _to_auth(row)
        return None

    def get_profile_by_device_user_id(self, env_url: str, user_id: str) -> Auth | None:
        """Get a profile by device OIDC user_id within an environment URL."""
        with sqlite3.connect(self.db_path) as conn:
            row = conn.execute(
                """
                SELECT id, name, api_url, project_id, api_key, api_key_id, device_oidc
                FROM profiles
                WHERE api_url = ? AND JSON_EXTRACT(device_oidc, '$.user_id') = ?
                LIMIT 1
                """,
                (env_url, user_id),
            ).fetchone()
            if row:
                return _to_auth(row)
        return None

    def list_profiles(self, env_url: str) -> list[Auth]:
        """List all profiles"""
        with sqlite3.connect(self.db_path) as conn:
            return [
                _to_auth(row)
                for row in conn.execute(
                    "SELECT id, name, api_url, project_id, api_key, api_key_id, device_oidc FROM profiles WHERE api_url = ? ORDER BY name",
                    (env_url,),
                ).fetchall()
            ]

    def delete_profile(self, name: str, env_url: str) -> bool:
        """Delete a profile by name. Returns True if deleted, False if not found."""
        with sqlite3.connect(self.db_path) as conn:
            cursor = conn.execute(
                "DELETE FROM profiles WHERE name = ? AND api_url = ?", (name, env_url)
            )
            conn.commit()

            # If this was the active profile, clear it
            if self.get_settings_current_profile_name() == name:
                self.set_settings_current_profile(None)

            return cursor.rowcount > 0

    def set_project(self, profile_name: str, env_url: str, project_id: str) -> bool:
        """Set the project for a profile. Returns True if profile exists."""
        with sqlite3.connect(self.db_path) as conn:
            cursor = conn.execute(
                "UPDATE profiles SET project_id = ? WHERE name = ? AND api_url = ?",
                (project_id, profile_name, env_url),
            )
            conn.commit()
            return cursor.rowcount > 0

    def update_profile(self, profile: Auth) -> None:
        """Update a profile"""
        with sqlite3.connect(self.db_path) as conn:
            conn.execute(
                "UPDATE profiles SET name = ?, api_url = ?, project_id = ?, api_key = ?, api_key_id = ?, device_oidc = ? WHERE id = ?",
                (
                    profile.name,
                    profile.api_url,
                    profile.project_id,
                    profile.api_key,
                    profile.api_key_id,
                    _serialize_device_oidc(profile.device_oidc),
                    profile.id,
                ),
            )
            conn.commit()

    ##################################
    ## Environments
    ##################################
    def create_or_update_environment(
        self, api_url: str, requires_auth: bool, min_llamactl_version: str | None = None
    ) -> None:
        """Create or update an environment row."""
        with sqlite3.connect(self.db_path) as conn:
            conn.execute(
                "INSERT OR REPLACE INTO environments (api_url, requires_auth, min_llamactl_version) VALUES (?, ?, ?)",
                (api_url, 1 if requires_auth else 0, min_llamactl_version),
            )
            conn.commit()

    def get_environment(self, api_url: str) -> Environment | None:
        """Retrieve an environment by URL."""
        with sqlite3.connect(self.db_path) as conn:
            row = conn.execute(
                "SELECT api_url, requires_auth, min_llamactl_version FROM environments WHERE api_url = ?",
                (api_url,),
            ).fetchone()
            if row:
                return _to_environment(row)
        return None

    def list_environments(self) -> list[Environment]:
        """List all environments."""
        with sqlite3.connect(self.db_path) as conn:
            envs = [
                _to_environment(row)
                for row in conn.execute(
                    "SELECT api_url, requires_auth, min_llamactl_version FROM environments ORDER BY api_url"
                ).fetchall()
            ]
            if not envs:
                envs = [DEFAULT_ENVIRONMENT]
            return envs

    def delete_environment(self, api_url: str) -> bool:
        """Delete an environment and all associated profiles.

        Returns True if the environment existed and was deleted, False otherwise.
        If the deleted environment was current, switch current to the default URL.
        """
        with sqlite3.connect(self.db_path) as conn:
            # Check existence
            exists_cursor = conn.execute(
                "SELECT 1 FROM environments WHERE api_url = ?",
                (api_url,),
            )
            if exists_cursor.fetchone() is None:
                return False

            # Delete profiles tied to this environment
            conn.execute("DELETE FROM profiles WHERE api_url = ?", (api_url,))

            # Delete environment row
            conn.execute("DELETE FROM environments WHERE api_url = ?", (api_url,))

            # If current environment is this one, reset to default
            setting_cursor = conn.execute(
                "SELECT value FROM settings WHERE key = 'current_environment_api_url'"
            )
            row = setting_cursor.fetchone()
            if row and row[0] == api_url:
                conn.execute(
                    "INSERT OR REPLACE INTO settings (key, value) VALUES ('current_environment_api_url', ?)",
                    (DEFAULT_ENVIRONMENT.api_url,),
                )

            conn.commit()
            return True


# Global config manager instance
@functools.cache
def config_manager() -> ConfigManager:
    return ConfigManager()



"""Textual-based deployment forms for CLI interactions"""

import dataclasses
import logging
import re
from dataclasses import dataclass, field
from pathlib import Path
from textwrap import dedent
from typing import cast
from urllib.parse import urlsplit

from llama_deploy.cli.client import get_project_client as get_client
from llama_deploy.cli.env import load_env_secrets_from_string
from llama_deploy.cli.textual.deployment_help import (
    DeploymentHelpBackMessage,
    DeploymentHelpWidget,
)
from llama_deploy.cli.textual.deployment_monitor import (
    DeploymentMonitorWidget,
    MonitorCloseMessage,
)
from llama_deploy.cli.textual.git_validation import (
    GitValidationWidget,
    ValidationCancelMessage,
    ValidationResultMessage,
)
from llama_deploy.cli.textual.secrets_form import SecretsWidget
from llama_deploy.cli.utils.version import get_installed_appserver_version
from llama_deploy.core.deployment_config import (
    DEFAULT_DEPLOYMENT_NAME,
    read_deployment_config,
)
from llama_deploy.core.git.git_util import (
    get_current_branch,
    get_git_root,
    get_unpushed_commits_count,
    is_git_repo,
    list_remotes,
    working_tree_has_changes,
)
from llama_deploy.core.schema.deployments import (
    DeploymentCreate,
    DeploymentResponse,
    DeploymentUpdate,
)
from packaging.version import Version
from textual import events
from textual.app import App, ComposeResult
from textual.containers import Container, HorizontalGroup
from textual.content import Content
from textual.message import Message
from textual.reactive import reactive
from textual.validation import Length
from textual.widget import Widget
from textual.widgets import Button, Input, Label, Select, Static


@dataclass
class DeploymentForm:
    """Form data for deployment editing/creation"""

    name: str = ""
    # unique id, generated from the name
    id: str | None = None
    repo_url: str = ""
    git_ref: str = "main"
    git_sha: str | None = None
    deployment_file_path: str = ""
    personal_access_token: str = ""
    # indicates if the deployment has a personal access token (value is unknown)
    has_existing_pat: bool = False
    # secrets that have been added
    secrets: dict[str, str] = field(default_factory=dict)
    # initial secrets, values unknown
    initial_secrets: set[str] = field(default_factory=set)
    # initial secrets that have been removed
    removed_secrets: set[str] = field(default_factory=set)
    # if the deployment is being edited
    is_editing: bool = False
    # warnings shown to the user
    warnings: list[str] = field(default_factory=list)
    # env info
    env_info_messages: str | None = None
    # appserver version fields
    installed_appserver_version: str | None = None
    existing_llama_deploy_version: str | None = None
    selected_appserver_version: str | None = None
    # required secret names from config
    required_secret_names: list[str] = field(default_factory=list)

    @classmethod
    def from_deployment(cls, deployment: DeploymentResponse) -> "DeploymentForm":
        secret_names = deployment.secret_names or []

        installed = get_installed_appserver_version()
        existing = deployment.llama_deploy_version
        # If versions match (or existing is None), treat as non-editable like create
        selected = existing or installed

        return DeploymentForm(
            name=deployment.name,
            id=deployment.id,
            repo_url=deployment.repo_url,
            git_ref=deployment.git_ref or "main",
            git_sha=deployment.git_sha or "-",
            deployment_file_path=deployment.deployment_file_path,
            personal_access_token="",  # Always start empty for security
            has_existing_pat=deployment.has_personal_access_token,
            secrets={},
            initial_secrets=set(secret_names),
            is_editing=True,
            installed_appserver_version=installed,
            existing_llama_deploy_version=existing,
            selected_appserver_version=selected,
        )

    @staticmethod
    def appserver_version() -> str | None:
        return get_installed_appserver_version()

    def to_update(self) -> DeploymentUpdate:
        """Convert form data to API format"""

        secrets: dict[str, str | None] = cast(
            # dict isn't covariant, so whatever, make it work
            dict[str, str | None],
            self.secrets.copy(),
        )
        for secret in self.removed_secrets:
            secrets[secret] = None

        appserver_version = self.selected_appserver_version

        data = DeploymentUpdate(
            repo_url=self.repo_url,
            git_ref=self.git_ref or "main",
            deployment_file_path=self.deployment_file_path or None,
            personal_access_token=(
                ""
                if self.personal_access_token is None and not self.has_existing_pat
                else self.personal_access_token
            ),
            secrets=secrets,
            llama_deploy_version=appserver_version,
        )

        return data

    def to_create(self) -> DeploymentCreate:
        """Convert form data to API format"""
        appserver_version = self.selected_appserver_version

        return DeploymentCreate(
            name=self.name,
            repo_url=self.repo_url,
            deployment_file_path=self.deployment_file_path or None,
            git_ref=self.git_ref or "main",
            personal_access_token=self.personal_access_token,
            secrets=self.secrets,
            llama_deploy_version=appserver_version,
        )


class DeploymentFormWidget(Widget):
    """Widget containing all deployment form logic and reactive state"""

    DEFAULT_CSS = """
    DeploymentFormWidget {
        layout: vertical;
        height: auto;
    }
    """

    form_data: reactive[DeploymentForm] = reactive(DeploymentForm(), recompose=True)
    error_message: reactive[str] = reactive("", recompose=True)

    def __init__(self, initial_data: DeploymentForm, save_error: str | None = None):
        super().__init__()
        self.form_data = initial_data
        self.original_form_data = initial_data
        self.error_message = save_error or ""

    def compose(self) -> ComposeResult:
        title = "Edit Deployment" if self.form_data.is_editing else "Create Deployment"

        with HorizontalGroup(
            classes="primary-message",
        ):
            yield Static(
                Content.from_markup(
                    f"{title} [italic][@click=app.show_help()]More info[/][/]"
                ),
                classes="w-1fr",
            )
            yield Static(
                Content.from_markup(
                    dedent("""
                [italic]Tab or click to navigate.[/]
                """).strip()
                ),
                classes="text-right w-1fr",
            )
        yield Static(
            self.error_message,
            id="error-message",
            classes="error-message " + ("visible" if self.error_message else "hidden"),
        )
        # Top-of-form warnings banner
        yield Static(
            "Note: " + " ".join(f"{w}" for w in self.form_data.warnings),
            id="warning-list",
            classes="warning-message mb-1 hidden "
            + ("visible" if self.form_data.warnings else ""),
        )

        # Main deployment fields
        with Widget(classes="two-column-form-grid"):
            yield Label(
                "Deployment Name: *", classes="required form-label", shrink=True
            )
            yield Input(
                value=self.form_data.name,
                placeholder="Enter deployment name",
                validators=[Length(minimum=1)],
                id="name",
                disabled=self.form_data.is_editing,
                classes="disabled" if self.form_data.is_editing else "",
                compact=True,
            )

            yield Label("Repository URL: *", classes="required form-label", shrink=True)
            yield Input(
                value=self.form_data.repo_url,
                placeholder="https://github.com/user/repo",
                validators=[Length(minimum=1)],
                id="repo_url",
                compact=True,
            )

            yield Label("Git Reference:", classes="form-label", shrink=True)
            yield Input(
                value=self.form_data.git_ref,
                placeholder="main, develop, v1.0.0, etc.",
                id="git_ref",
                compact=True,
            )

            yield Label("Last Deployed Commit:", classes="form-label", shrink=True)
            yield Input(
                value=(self.form_data.git_sha or "-")[:7],
                placeholder="-",
                id="git_sha",
                compact=True,
                disabled=True,
            )

            yield Static(classes="full-width")
            yield Static(
                Content.from_markup("[italic]Advanced[/]"),
                classes="text-center full-width",
            )
            yield Label("Config File:", classes="form-label", shrink=True)
            yield Input(
                value=self.form_data.deployment_file_path,
                placeholder="Optional path to config dir/file",
                id="deployment_file_path",
                compact=True,
            )

            yield Label("Personal Access Token:", classes="form-label", shrink=True)
            if self.form_data.has_existing_pat:
                yield Button(
                    "Change / Delete",
                    variant="default",
                    id="change_pat",
                    compact=True,
                )
            else:
                yield Input(
                    value=self.form_data.personal_access_token,
                    placeholder="Leave blank to clear"
                    if self.form_data.has_existing_pat
                    else "Optional",
                    password=True,
                    id="personal_access_token",
                    compact=True,
                )

            # Appserver version display/selector
            yield Label("Appserver Version:", classes="form-label", shrink=True)
            versions_differ = (
                self.form_data.is_editing
                and self.form_data.installed_appserver_version
                and self.form_data.existing_llama_deploy_version
                and self.form_data.installed_appserver_version
                != self.form_data.existing_llama_deploy_version
            )
            if versions_differ:
                # Show dropdown selector for version choice
                installed_version = self.form_data.installed_appserver_version
                existing_version = self.form_data.existing_llama_deploy_version
                current_selection = (
                    self.form_data.selected_appserver_version
                    or existing_version
                    or installed_version
                )
                is_upgrade = (
                    installed_version
                    and existing_version
                    and Version(installed_version) > Version(existing_version)
                )
                label = "Upgrade" if is_upgrade else "Downgrade"
                yield Select(
                    [
                        (f"{label} to {installed_version}", installed_version),
                        (f"Keep {existing_version}", existing_version),
                    ],
                    value=current_selection,
                    id="appserver_version_select",
                    allow_blank=False,
                    compact=True,
                )
            else:
                # Non-editable display of version
                readonly_version = (
                    self.form_data.installed_appserver_version
                    or self.form_data.existing_llama_deploy_version
                    or "unknown"
                )
                yield Static(readonly_version, id="appserver_version_readonly")

        # Secrets section
        yield SecretsWidget(
            initial_secrets=self.form_data.secrets,
            prior_secrets=self.form_data.initial_secrets,
            info_message=self.form_data.env_info_messages,
        )

        with HorizontalGroup(classes="button-row"):
            yield Button("Save", variant="primary", id="save", compact=True)
            yield Button("Cancel", variant="default", id="cancel", compact=True)

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "save":
            # Ensure latest input values are captured by blurring current focus first
            try:
                if self.screen.focused is not None:
                    self.screen.focused.blur()
            except Exception:
                pass
            self._save()
        elif event.button.id == "change_pat":
            updated_form = dataclasses.replace(self.resolve_form_data())
            updated_form.has_existing_pat = False
            updated_form.personal_access_token = ""
            self.form_data = updated_form
        elif event.button.id == "cancel":
            # Post message to parent app to handle cancel
            self.post_message(CancelFormMessage())

    def on_select_changed(self, event: Select.Changed) -> None:
        """Handle version selection changes"""
        if event.select.id == "appserver_version_select" and event.value:
            updated_form = dataclasses.replace(self.resolve_form_data())
            updated_form.selected_appserver_version = str(event.value)
            self.form_data = updated_form

    def _save(self) -> None:
        self.form_data = self.resolve_form_data()
        if self._validate_form():
            # Post message to parent app to start validation
            self.post_message(StartValidationMessage(self.form_data))

    def _validate_form(self) -> bool:
        """Validate required fields from the current UI state"""
        name_input = self.query_one("#name", Input)
        repo_url_input = self.query_one("#repo_url", Input)

        errors: list[str] = []

        # Clear previous error state
        name_input.remove_class("error")
        repo_url_input.remove_class("error")

        if not name_input.value.strip():
            name_input.add_class("error")
            errors.append("Deployment name is required")

        if not repo_url_input.value.strip():
            repo_url_input.add_class("error")
            errors.append("Repository URL is required")

        missing_required: list[str] = []
        for secret_name in sorted(self.form_data.required_secret_names):
            value = (self.form_data.secrets.get(secret_name) or "").strip()
            if value == "":
                missing_required.append(secret_name)
        if missing_required:
            errors.append("Missing required secrets: " + ", ".join(missing_required))

        if errors:
            self._show_error("; ".join(errors))
            return False
        self._show_error("")
        return True

    def _show_error(self, message: str) -> None:
        """Show an error message"""
        self.error_message = message

    def resolve_form_data(self) -> DeploymentForm:
        """Extract form data from inputs"""
        name_input = self.query_one("#name", Input)
        repo_url_input = self.query_one("#repo_url", Input)
        git_ref_input = self.query_one("#git_ref", Input)
        deployment_file_input = self.query_one("#deployment_file_path", Input)

        # PAT input might not exist if there's an existing PAT
        try:
            pat_input = self.query_one("#personal_access_token", Input)
            pat_value = pat_input.value.strip()
        except Exception:
            pat_value = self.form_data.personal_access_token or ""

        # Get updated secrets from the secrets widget
        secrets_widget = self.query_one(SecretsWidget)
        updated_secrets = secrets_widget.get_updated_secrets()
        updated_prior_secrets = secrets_widget.get_updated_prior_secrets()

        return DeploymentForm(
            name=name_input.value.strip(),
            id=self.form_data.id,
            repo_url=repo_url_input.value.strip(),
            git_ref=git_ref_input.value.strip() or "main",
            deployment_file_path=deployment_file_input.value.strip(),
            personal_access_token=pat_value,
            secrets=updated_secrets,
            initial_secrets=self.original_form_data.initial_secrets,
            is_editing=self.original_form_data.is_editing,
            has_existing_pat=self.form_data.has_existing_pat,
            removed_secrets=self.original_form_data.initial_secrets.difference(
                updated_prior_secrets
            ),
            installed_appserver_version=self.form_data.installed_appserver_version,
            existing_llama_deploy_version=self.form_data.existing_llama_deploy_version,
            selected_appserver_version=self.form_data.selected_appserver_version,
            required_secret_names=self.form_data.required_secret_names,
        )


# Messages for communication between form widget and app
class SaveFormMessage(Message):
    def __init__(self, deployment: DeploymentResponse):
        super().__init__()
        self.deployment = deployment


class CancelFormMessage(Message):
    pass


class StartValidationMessage(Message):
    def __init__(self, form_data: DeploymentForm):
        super().__init__()
        self.form_data = form_data


class ShowHelpMessage(Message):
    def __init__(self, form_data: DeploymentForm):
        super().__init__()
        self.form_data = form_data


class HelpBackMessage(Message):
    pass


class DeploymentEditApp(App[DeploymentResponse | None]):
    """Textual app for editing/creating deployments"""

    CSS_PATH = Path(__file__).parent / "styles.tcss"

    # App states: 'form', 'validation', 'help', or 'monitor'
    current_state: reactive[str] = reactive("form", recompose=True)
    form_data: reactive[DeploymentForm] = reactive(DeploymentForm())
    save_error: reactive[str] = reactive("", recompose=True)
    saved_deployment = reactive[DeploymentResponse | None](None, recompose=True)

    def __init__(self, initial_data: DeploymentForm):
        super().__init__()
        self.initial_data = initial_data
        self.form_data = initial_data

    def on_mount(self) -> None:
        self.theme = "tokyo-night"

    def on_key(self, event: events.Key) -> None:
        """Handle key events, including Ctrl+C"""
        if event.key == "ctrl+c":
            if self.current_state == "monitor" and self.saved_deployment is not None:
                self.exit(self.saved_deployment)
            else:
                self.exit(None)

    def compose(self) -> ComposeResult:
        is_slim = self.current_state != "monitor"
        with Container(classes="form-container" if is_slim else ""):
            if self.current_state == "form":
                yield DeploymentFormWidget(self.form_data, self.save_error)
            elif self.current_state == "validation":
                yield GitValidationWidget(
                    repo_url=self.form_data.repo_url,
                    deployment_id=self.form_data.id
                    if self.form_data.is_editing
                    else None,
                    pat=self.form_data.personal_access_token
                    if self.form_data.personal_access_token
                    else None,
                )
            elif self.current_state == "help":
                yield DeploymentHelpWidget()
            elif self.current_state == "monitor":
                deployment_id = (
                    self.saved_deployment.id if self.saved_deployment else ""
                )
                yield DeploymentMonitorWidget(deployment_id)
            else:
                yield Static("Unknown state: " + self.current_state)

    def action_show_help(self) -> None:
        widget = self.query("DeploymentFormWidget")
        if widget:
            first_widget = widget[0]
            if isinstance(first_widget, DeploymentFormWidget):
                self.form_data = first_widget.resolve_form_data()

        self.current_state = "help"

    def on_deployment_help_back_message(
        self, message: DeploymentHelpBackMessage
    ) -> None:
        self.current_state = "form"

    def on_start_validation_message(self, message: StartValidationMessage) -> None:
        """Handle validation start message from form widget"""
        self.form_data = message.form_data
        self.save_error = ""  # Clear any previous errors
        self.current_state = "validation"

    async def on_validation_result_message(
        self, message: ValidationResultMessage
    ) -> None:
        """Handle validation success from git validation widget"""
        logging.info("validation result message", message)
        # Update form data with validated PAT if provided
        if message.pat is not None:
            updated_form = dataclasses.replace(self.form_data)
            updated_form.personal_access_token = message.pat
            # If PAT is being cleared (empty string), also clear the has_existing_pat flag
            if message.pat == "":
                updated_form.has_existing_pat = False
            self.form_data = updated_form

        # Proceed with save (async)
        await self._perform_save()

    def on_validation_cancel_message(self, message: ValidationCancelMessage) -> None:
        """Handle validation cancellation from git validation widget"""
        # Return to form, clearing any save error
        self.save_error = ""
        self.current_state = "form"

    def on_show_help_message(self, message: ShowHelpMessage) -> None:
        """Navigate to help view, preserving current form state."""
        self.form_data = message.form_data
        self.current_state = "help"

    def on_help_back_message(self, message: HelpBackMessage) -> None:
        """Return from help to form, keeping form state intact."""
        self.current_state = "form"

    async def _perform_save(self) -> None:
        """Actually save the deployment after validation"""
        logging.info("saving form data", self.form_data)
        result = self.form_data
        client = get_client()
        try:
            if result.is_editing:
                if not result.id:
                    raise ValueError("Deployment ID is required for update")
                update_deployment = await client.update_deployment(
                    result.id, result.to_update()
                )
            else:
                update_deployment = await client.create_deployment(result.to_create())
            # Save and navigate to embedded monitor screen
            self.saved_deployment = update_deployment
            # Ensure form_data carries the new ID for any subsequent operations
            if not result.is_editing and update_deployment.id:
                updated_form = dataclasses.replace(self.form_data)
                updated_form.id = update_deployment.id
                updated_form.is_editing = True
                self.form_data = updated_form
            self.current_state = "monitor"
        except Exception as e:
            # Return to form and show informative error
            self.save_error = f"Error saving deployment: {e}"
            self.current_state = "form"

    def on_save_form_message(self, message: SaveFormMessage) -> None:
        """Handle save message from form widget (shouldn't happen with new flow)"""
        self.exit(message.deployment)

    def on_cancel_form_message(self, message: CancelFormMessage) -> None:
        """Handle cancel message from form widget"""
        self.exit(None)

    def on_monitor_close_message(self, _: MonitorCloseMessage) -> None:
        """Handle close from embedded monitor by exiting with saved deployment."""
        self.exit(self.saved_deployment)


def edit_deployment_form(
    deployment: DeploymentResponse,
) -> DeploymentResponse | None:
    """Launch deployment edit form and return result"""
    initial_data = DeploymentForm.from_deployment(deployment)
    app = DeploymentEditApp(initial_data)
    return app.run()


def create_deployment_form() -> DeploymentResponse | None:
    """Launch deployment creation form and return result"""
    initial_data = _initialize_deployment_data()
    app = DeploymentEditApp(initial_data)
    return app.run()


def _initialize_deployment_data() -> DeploymentForm:
    """
    initialize the deployment form data from the current git repo and .env file
    """

    repo_url: str | None = None
    git_ref: str | None = None
    secrets: dict[str, str] = {}
    name: str | None = None
    config_file_path: str | None = None
    warnings: list[str] = []
    has_git = is_git_repo()
    has_no_workflows = False
    required_secret_names: list[str] = []
    try:
        config = read_deployment_config(Path("."), Path("."))
        if config.name != DEFAULT_DEPLOYMENT_NAME:
            name = config.name
        has_no_workflows = config.has_no_workflows()
        # Seed required secret names from config if present
        required_secret_names = config.required_env_vars

    except Exception:
        warnings.append("Could not parse local deployment config. It may be invalid.")
    if not has_git and has_no_workflows:
        warnings = [
            "Run from within a git repository to automatically generate a deployment config."
        ]
    elif has_no_workflows:
        warnings = [
            "The current project has no workflows configured. It may be invalid."
        ]
    elif not has_git:
        warnings.append(
            "Current directory is not a git repository. If you are trying to deploy this directory, you will need to create a git repository and push it before creating a deployment."
        )
    else:
        seen = set[str]()
        remotes = list_remotes()
        candidate_origins = []
        for remote in remotes:
            normalized_url = _normalize_to_http(remote)
            if normalized_url not in seen:
                candidate_origins.append(normalized_url)
                seen.add(normalized_url)
        preferred_origin = sorted(
            candidate_origins, key=lambda x: "github.com" in x, reverse=True
        )
        if preferred_origin:
            repo_url = preferred_origin[0]
        git_ref = get_current_branch()
        root = get_git_root()
        if root != Path.cwd():
            config_file_path = str(Path.cwd().relative_to(root))

        if not preferred_origin:
            warnings.append(
                "No git remote was found. You will need to push your changes to a remote repository before creating a deployment from this repository."
            )
        else:
            # Working tree changes
            if working_tree_has_changes() and preferred_origin:
                warnings.append(
                    "Working tree has uncommitted or untracked changes. You may want to push them before creating a deployment from this branch."
                )
            else:
                # Unpushed commits (ahead of upstream)
                ahead = get_unpushed_commits_count()
                if ahead is None:
                    warnings.append(
                        "Current branch has no upstream configured. You will need to push them or choose a different branch."
                    )
                elif ahead > 0:
                    warnings.append(
                        f"There are {ahead} local commits not pushed to upstream. They won't be included in the deployment unless you push them first."
                    )
    env_info_message = None
    if Path(".env").exists():
        secrets = load_env_secrets_from_string(Path(".env").read_text())
        if len(secrets) > 0:
            env_info_message = "Secrets were automatically seeded from your .env file. Remove or change any that should not be set. They must be manually configured after creation."

    installed = get_installed_appserver_version()

    form = DeploymentForm(
        name=name or "",
        repo_url=repo_url or "",
        git_ref=git_ref or "main",
        secrets=secrets,
        deployment_file_path=config_file_path or "",
        warnings=warnings,
        env_info_messages=env_info_message,
        installed_appserver_version=installed,
        selected_appserver_version=installed,
        required_secret_names=required_secret_names,
    )
    return form


def _normalize_to_http(url: str) -> str:
    """
    normalize a git url to a best guess for a corresponding http(s) url
    """
    candidate = (url or "").strip()

    # If no scheme, first try scp-like SSH syntax: [user@]host:path
    has_scheme = "://" in candidate
    if not has_scheme:
        scp_match = re.match(
            r"^(?:(?P<user>[^@]+)@)?(?P<host>[^:/\s]+):(?P<path>[^/].+)$",
            candidate,
        )
        if scp_match:
            host = scp_match.group("host")
            path = scp_match.group("path").lstrip("/")
            if path.endswith(".git"):
                path = path[:-4]
            return f"https://{host}/{path}"

    # If no scheme (and not scp), assume host/path and prepend https
    parsed = urlsplit(candidate if has_scheme else f"https://{candidate}")

    # Drop credentials from netloc
    netloc = parsed.netloc.split("@", 1)[-1]

    # Drop explicit port (common for SSH like :7999 which is wrong for https)
    if ":" in netloc:
        netloc = netloc.split(":", 1)[0]

    # Normalize path and strip .git
    path = parsed.path.lstrip("/")
    if path.endswith(".git"):
        path = path[:-4]

    if path:
        return f"https://{netloc}/{path}"
    else:
        return f"https://{netloc}"



from __future__ import annotations

import asyncio
import logging
import os
from pathlib import Path
from typing import TYPE_CHECKING, Literal

import click
from click.exceptions import Abort, Exit
from llama_deploy.cli.commands.auth import validate_authenticated_profile
from llama_deploy.cli.options import (
    interactive_option,
    native_tls_option,
)
from llama_deploy.cli.styles import WARNING
from llama_deploy.cli.utils.redact import redact_api_key
from llama_deploy.core.config import DEFAULT_DEPLOYMENT_FILE_PATH
from llama_deploy.core.deployment_config import (
    read_deployment_config_from_git_root_or_cwd,
)
from llama_deploy.core.schema.projects import ProjectSummary
from rich import print as rprint

from ..app import app

if TYPE_CHECKING:
    from llama_deploy.cli.config.schema import Auth

logger = logging.getLogger(__name__)
_ClickPath = getattr(click, "Path")


@app.command(
    "serve",
    help="Serve a LlamaDeploy app locally for development and testing",
)
@click.argument(
    "deployment_file",
    required=False,
    default=DEFAULT_DEPLOYMENT_FILE_PATH,
    type=_ClickPath(dir_okay=True, resolve_path=True, path_type=Path),
)
@click.option(
    "--no-install", is_flag=True, help="Skip installing python and js dependencies"
)
@click.option(
    "--no-reload", is_flag=True, help="Skip reloading the API server on code changes"
)
@click.option("--no-open-browser", is_flag=True, help="Skip opening the browser")
@click.option(
    "--preview",
    is_flag=True,
    help="Preview mode pre-builds the UI to static files, like a production build",
)
@click.option("--port", type=int, help="The port to run the API server on")
@click.option("--ui-port", type=int, help="The port to run the UI proxy server on")
@click.option(
    "--log-level",
    type=click.Choice(
        ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"], case_sensitive=False
    ),
    help="The log level to run the API server at",
)
@click.option(
    "--log-format",
    type=click.Choice(["console", "json"], case_sensitive=False),
    help="The format to use for logging",
)
@click.option(
    "--persistence",
    type=click.Choice(["memory", "local", "cloud"]),
    help="The persistence mode to use for the workflow server",
)
@click.option(
    "--local-persistence-path",
    type=_ClickPath(dir_okay=True, resolve_path=True, path_type=Path),
    help="The path to the sqlite database to use for the workflow server if using local persistence",
)
@click.option(
    "--host",
    type=str,
    help="The host to run the API server on. Default is 127.0.0.1. Use 0.0.0.0 to allow remote access.",
)
@interactive_option
@native_tls_option
def serve(
    deployment_file: Path,
    no_install: bool,
    no_reload: bool,
    no_open_browser: bool,
    preview: bool,
    port: int | None = None,
    ui_port: int | None = None,
    log_level: str | None = None,
    log_format: str | None = None,
    persistence: Literal["memory", "local", "cloud"] | None = None,
    local_persistence_path: Path | None = None,
    host: str | None = None,
    interactive: bool = False,
) -> None:
    """Run llama_deploy API Server in the foreground. Reads the deployment configuration from the current directory. Can optionally specify a deployment file path."""
    if not deployment_file.exists():
        rprint(f"[red]Deployment file '{deployment_file}' not found[/red]")
        raise click.Abort()

    # Early check: appserver requires a pyproject.toml in the config directory
    config_dir = deployment_file if deployment_file.is_dir() else deployment_file.parent
    if not (config_dir / "pyproject.toml").exists():
        rprint(
            "[red]No pyproject.toml found at[/red] "
            f"[bold]{config_dir}[/bold].\n"
            "Add a pyproject.toml to your project and re-run 'llamactl serve'."
        )
        raise click.Abort()

    try:
        # Pre-check: if the template requires llama cloud access, ensure credentials
        _maybe_inject_llama_cloud_credentials(
            deployment_file, interactive, require_cloud=persistence == "cloud"
        )

        # Defer heavy appserver imports until the `serve` command is actually invoked
        from llama_deploy.appserver.app import (
            prepare_server,
            start_server_in_target_venv,
        )
        from llama_deploy.appserver.deployment_config_parser import (
            get_deployment_config,
        )

        prepare_server(
            deployment_file=deployment_file,
            install=not no_install,
            build=preview,
        )
        deployment_config = get_deployment_config()
        _print_connection_summary()
        start_server_in_target_venv(
            cwd=Path.cwd(),
            deployment_file=deployment_file,
            proxy_ui=not preview,
            reload=not no_reload,
            open_browser=not no_open_browser,
            port=port,
            ui_port=ui_port,
            log_level=log_level.upper() if log_level else None,
            log_format=log_format.lower() if log_format else None,
            persistence=persistence if persistence else "local",
            local_persistence_path=str(local_persistence_path)
            if local_persistence_path and persistence == "local"
            else None,
            cloud_persistence_name=f"_public:serve_workflows_{deployment_config.name}"
            if persistence == "cloud"
            else None,
            host=host,
        )

    except (Exit, Abort):
        raise

    except KeyboardInterrupt:
        logger.debug("Shutting down...")

    except Exception as e:
        rprint(f"[red]Error: {e}[/red]")
        raise click.Abort()


def _set_env_vars_from_profile(profile: Auth) -> None:
    if profile.api_key:
        _set_env_vars(profile.api_key, profile.api_url)
    _set_project_id_from_profile(profile)


def _set_env_vars_from_env(env_vars: dict[str, str]) -> None:
    key = env_vars.get("LLAMA_CLOUD_API_KEY")
    url = env_vars.get("LLAMA_CLOUD_BASE_URL", "https://api.cloud.llamaindex.ai")
    # Also propagate project id if present in the environment
    _set_project_id_from_env(env_vars)
    if key:
        _set_env_vars(key, url)


def _set_env_vars(key: str, url: str) -> None:
    os.environ["LLAMA_CLOUD_API_KEY"] = key
    os.environ["LLAMA_CLOUD_BASE_URL"] = url
    # kludge for common web servers to inject local auth key
    for prefix in ["VITE_", "NEXT_PUBLIC_"]:
        os.environ[f"{prefix}LLAMA_CLOUD_API_KEY"] = key
        os.environ[f"{prefix}LLAMA_CLOUD_BASE_URL"] = url


def _set_project_id_from_env(env_vars: dict[str, str]) -> None:
    project_id = env_vars.get("LLAMA_DEPLOY_PROJECT_ID")
    if project_id:
        os.environ["LLAMA_DEPLOY_PROJECT_ID"] = project_id


def _set_project_id_from_profile(profile: Auth) -> None:
    if profile.project_id:
        os.environ["LLAMA_DEPLOY_PROJECT_ID"] = profile.project_id


def _maybe_inject_llama_cloud_credentials(
    deployment_file: Path, interactive: bool, require_cloud: bool
) -> None:
    """If the deployment config indicates Llama Cloud usage, ensure LLAMA_CLOUD_API_KEY is set.

    Behavior:
    - If LLAMA_CLOUD_API_KEY is already set, do nothing.
    - Else, try to read current profile's api_key and inject.
    - If no profile/api_key and session is interactive, prompt to log in and inject afterward.
    - If user declines or session is non-interactive, warn that deployment may not work.
    """
    import questionary
    from llama_deploy.appserver.workflow_loader import parse_environment_variables
    from llama_deploy.cli.config.env_service import service

    # Read config directly to avoid cached global settings
    try:
        config = read_deployment_config_from_git_root_or_cwd(
            Path.cwd(), deployment_file
        )
    except Exception:
        rprint(
            "[red]Error: Could not read a deployment config. This doesn't appear to be a valid llama-deploy project.[/red]"
        )
        raise click.Abort()

    if not config.llama_cloud and not require_cloud:
        return

    vars = parse_environment_variables(
        config, deployment_file.parent if deployment_file.is_file() else deployment_file
    )

    # Ensure project id is available to the app and UI processes
    _set_project_id_from_env({**os.environ, **vars})

    existing = os.environ.get("LLAMA_CLOUD_API_KEY") or vars.get("LLAMA_CLOUD_API_KEY")
    if existing:
        # If interactive, allow choosing between env var and configured profile
        if interactive:
            choice = questionary.select(
                "LLAMA_CLOUD_API_KEY detected in environment. Which credentials do you want to use?",
                choices=[
                    questionary.Choice(
                        title=f"Use environment variable - {redact_api_key(existing)}",
                        value="env",
                    ),
                    questionary.Choice(title="Use configured profile", value="profile"),
                ],
            ).ask()
            if choice is None:
                raise Exit(0)
            if choice == "profile":
                # Ensure we have an authenticated profile and inject from it
                authed = validate_authenticated_profile(True)
                _set_env_vars_from_profile(authed)
                return
            # Default to env var path when cancelled or explicitly chosen
            _set_env_vars_from_env({**os.environ, **vars})
            # If no project id provided, try to detect and select one using the env API key
            if not os.environ.get("LLAMA_DEPLOY_PROJECT_ID"):
                _maybe_select_project_for_env_key()
            return
        # Non-interactive: trust current environment variables
        _set_env_vars_from_env({**os.environ, **vars})
        return

    env = service.get_current_environment()
    if not env.requires_auth:
        rprint(
            f"[{WARNING}]Warning: This app requires Llama Cloud authentication, and no LLAMA_CLOUD_API_KEY is present. The app may not work.[/]"
        )
        return

    auth_svc = service.current_auth_service()
    profile = auth_svc.get_current_profile()
    if profile and profile.api_key:
        _set_env_vars_from_profile(profile)
        return

    # No key available; consider prompting if interactive
    if interactive:
        should_login = questionary.confirm(
            "This deployment requires Llama Cloud. Login now to inject credentials? Otherwise the app may not work.",
            default=True,
        ).ask()
        if should_login:
            authed = validate_authenticated_profile(True)
            if authed.api_key:
                _set_env_vars_from_profile(authed)
                return
        rprint(
            f"[{WARNING}]Warning: No Llama Cloud credentials configured. The app may not work.[/]"
        )
        return

    # Non-interactive session
    rprint(
        f"[{WARNING}]Warning: LLAMA_CLOUD_API_KEY is not set and no logged-in profile was found. The app may not work.[/]"
    )


def _maybe_select_project_for_env_key() -> None:
    """When using an env API key, ensure LLAMA_DEPLOY_PROJECT_ID is set.

    If more than one project exists, prompt the user to select one.
    """
    import questionary
    from llama_deploy.core.client.manage_client import ControlPlaneClient

    api_key = os.environ.get("LLAMA_CLOUD_API_KEY")
    base_url = os.environ.get("LLAMA_CLOUD_BASE_URL", "https://api.cloud.llamaindex.ai")
    if not api_key:
        return
    try:

        async def _run() -> list[ProjectSummary]:
            async with ControlPlaneClient.ctx(base_url, api_key, None) as client:
                return await client.list_projects()

        projects = asyncio.run(_run())
        if not projects:
            return
        if len(projects) == 1:
            os.environ["LLAMA_DEPLOY_PROJECT_ID"] = projects[0].project_id
            return
        # Multiple: prompt selection
        choice = questionary.select(
            "Select a project",
            choices=[
                questionary.Choice(
                    title=f"{p.project_name} ({p.deployment_count} deployments)",
                    value=p.project_id,
                )
                for p in projects
            ],
        ).ask()
        if choice:
            os.environ["LLAMA_DEPLOY_PROJECT_ID"] = choice
    except Exception:
        # Best-effort; if we fail to list, do nothing
        pass


def _print_connection_summary() -> None:
    base_url = os.environ.get("LLAMA_CLOUD_BASE_URL")
    project_id = os.environ.get("LLAMA_DEPLOY_PROJECT_ID")
    api_key = os.environ.get("LLAMA_CLOUD_API_KEY")
    if not base_url and not project_id and not api_key:
        return
    redacted = redact_api_key(api_key)
    env_text = base_url or "-"
    proj_text = project_id or "-"
    rprint(
        f"Connecting to environment: [bold]{env_text}[/], project: [bold]{proj_text}[/], api key: [bold]{redacted}[/]"
    )
